/*
MODEL.H file for the CUBICSYMMBREAK model.
Written by Gary Felder (gfelder@physics.stanford.edu)
Last Modified 3/2/01

This file contains the model specific functions and definitions for the model:
V = 1/4 lambda phi^4 - 1/3 lambda v phi^3 + 1/12 lambda v^4
Comments: This model is for investigating symmetry breaking in a simple, one-field case. The field phi is assumed to begin at zero and roll down from there to its minimum at +-v.

The potential in program units is given by V_{pr} = 1/4 phi_{pr}^2 - 1/3 phi_{pr}^3 + 1/12
*/

/*
General comments about the model.h file:
This file contains the following functions - all called externally
modelinfo(FILE *info_) outputs information about the model and model-specific parameters to a file.
modelinitialize() performs any model-specific initialization
potential_energy(int term, float *field_values) calculates the average potential energy density term by term. The variable num_potential_terms (just above this function) specifies how many separate potential terms are used in this model.
dvdf(int fld, int i, int j, int k) calculates the potential term in the equation of motion, dV/dfield, for the field fld at the lattice point (i,j,k)
effective_mass(float mass_sq[], float *field_values) calculates the square masses of the fields and puts them in the array mass_sq. The parameter beginning tells the function to use initial field values - if this parameter is zero then the field quantities will be calculated dynamically.
model_output(int flush, char *ext_) allows each model to include its own specialized output function(s). The parameter flush is set to 1 when infrequent calculations are being performed and 0 otherwise. The string ext_ gives the extension for output filenames.
*/

/* - This section should be copied into parameters.h when using this model
// ---Adjustable parameters for CUBICSYMMBREAK model--- //
const float lambda=1.e-1; // Self-coupling of inflaton
const float v=1.e-3; // Symmetry breaking scale
const float finit=pi*pw2(v); // Iniitial value of the field phi
const float fdinit=sqrt(2.*pi*pi*pi)*v*v*v; // Initial value of the derivative phi'
*/

// Rescaling parameters.
// The program variables ("_pr") are defined as
//   f_pr = rescale_A a^rescale_r f (f=field value)
//   x_pr = rescale_B x (x=distance)
//   dt_pr = rescale_B a^rescale_s dt (t=time)
// The constants beta, cpl, and f0 are used to set the variable rescalings rescale_A, rescale_B, rescale_r, and rescale_s.
// These rescaling constants may be reset independently; their settings in terms of beta, cpl, and f0 are suggestions. See the documentation for more details.
// These rescalings are intrinsic to the model and probably shouldn't be changed for individual runs. Adjustable parameters are stored in "parameters.h"
const float beta=3.; // Exponent of the dominant term in the potential
const float cpl=lambda*v; // Coefficient of the dominant term in the potential (up to numerical factors - see documentation)
const float f0=v; // Initial value of phi in Planck units, typically the point at which phi'=0
// By default these are automatically set to A=1/f0, B=sqrt(cpl) f0^(-1+beta/2), R=6/(2+beta), S=3(2-beta)/(2+beta). They may be adjusted to different values, but the relationship S=2R-3 must be maintained for the program equations to remain correct.
const float rescale_A=1./v;
const float rescale_B=sqrt(lambda)*v;
const float rescale_r=1.5;
// The value of S in terms of R SHOULD NOT be changed.
const float rescale_s=2.*rescale_r-3.;

// Other global variables
// The array model_vars is intended to hold any model-specific, non-constant global variables.
// These variables should be initialized in modelinitialize() below
// Even if you're not using any, num_model_vars should be at least 1 to keep some compilers happy.
const int num_model_vars=1;
// Model specific variables:
//   0: Renormalization of source term
extern float model_vars[num_model_vars];

// Macros to make the equations more readable: The only allowed value of fld is 0=Phi
#define PHI FIELD(0)
#define SOURCETERM model_vars[0]
#define A3 (1./(a*a*a))
#define A32 (sqrt(A3))

// Model specific details about the run to be output to an information file
inline void modelinfo(FILE *info_)
{
  // Name and description of model
  fprintf(info_,"Cubic Symmetery Breaking Model\n");
  fprintf(info_,"V = 1/4 lambda phi^4 - 1/3 lambda v phi^3 + 1/12 lambda v^4\n\n");

  // Model specific parameter values
  fprintf(info_,"lambda = %e\n",lambda);
  fprintf(info_,"v = %e\n",v);
}

// Perform any model specific initialization
// This function is called twice, once before initializing the fields (which_call=1) and once after (which_call=2)
inline void modelinitialize(int which_call)
{
  if(which_call==1)
  {
    if(nflds!=1)
    {
      printf("Number of fields for CUBICSYMMBREAK model must be 1. Exiting.\n");
      exit(1);
    }

    // Calculate renormalization of source term
    SOURCETERM=0.;
    int i,j,k;
    float index;
    float indexmax=L*kcutoff/2./pi;
    if(kcutoff==0.)
      indexmax=100.*(float)N/L; // A number bigger than the biggest value of i on the lattice

    if(NDIMS==1)
    {
      for(i=-N/2;i<N/2;i++)
      {
        index = (float)abs(i);
        if(index <= indexmax)
          SOURCETERM += index;
      }
    }
    else if(NDIMS==2)
    {
      for(i=-N/2;i<N/2;i++)
        for(j=-N/2;j<N/2;j++)
        {
          index = sqrt((float)(i*i+j*j));
          if(index <= indexmax)
            SOURCETERM += 1.;
        }
        SOURCETERM /= pi;
    }
    else if(NDIMS==3)
    {
      for(i=-N/2;i<N/2;i++)
        for(j=-N/2;j<N/2;j++)
          for(k=-N/2;k<N/2;k++)
          {
            index = sqrt((float)(i*i+j*j+k*k));
            if(index>0. && index <= indexmax)
              SOURCETERM += 1./index;
          }
      SOURCETERM /= 2.*pi;
    }

    SOURCETERM *= pw2(rescale_A*rescale_B)/2./pw2(L);
  }
}

// The constant num_potential_terms must be defined for use by outside functions
// Terms: term=0: 1/4 lambda phi^4 --- term=1: 1/3 lambda v phi^3 --- term=2: 1/12 lambda v^4
const int num_potential_terms=3; // Number of terms that are calculated separately in the potential
// Potential energy terms
// See documentation for normalization of these terms.
// When setting initial conditions field values will be supplied in the array field_values. Otherwise this function will calculate them on the lattice.
inline float potential_energy(int term, float *field_values)
{
  DECLARE_INDICES
  float potential=0.;

  if(field_values==NULL) // If no values are given calculate averages on the lattice
  {
    // Loop over grid to calculate potential term
    LOOP
    {
      if(term==0)
        potential += pw2(pw2(PHI));
      else if(term==1)
        potential += PHI*pw2(PHI);
    }
        
    // Convert sum to average
    potential /= gridsize;
  }
  else // If field values are given then use them instead
  {
    if(term==0)
      potential = pw2(pw2(field_values[0]));
    else if(term==1)
      potential = field_values[0]*pw2(field_values[0]);
  }

  // Include numerical coefficients
  if(term==0)
    potential *= .25*A3;
  else if(term==1)
    potential *= -.333*A32;
  else
    potential = 1./12./A3;

  return (potential);
}

// Potential terms in the equations of motion, dV/dfield, evaluated at point (i,j,k)
// See documentation for details on the normalization of these terms
inline float dvdf(int fld, INDEXLIST)
{
  static float a3=A3,a32=A32; // Since this same value get used at every point only calculate it once per time
  static float tprevious = t;
  if(t!=tprevious)
  {
    a3=A3;
    a32=A32;
    tprevious=t;
  }

  return( (a3*PHI - a32)*pw2(PHI) + a*SOURCETERM );
}

// Calculate effective mass squared and put it into the array mass_sq[] (used for initial conditions and power spectra)
// See documentation for normalization of these terms
// When setting initial conditions field values will be supplied in the array field_values. Otherwise this function will calculate them on the lattice.
inline void effective_mass(float mass_sq[], float *field_values)
{
  DECLARE_INDICES
  int fld;
  float fldsqrd[nflds]; // Square value of field
  float fldvalue[nflds]; // Average value of field
  float correction; // Used to adjust masses by the appropriate power of the scale factor. (See documentation.)
  
  // Loop over fields to find mean-square value
  if(field_values==NULL) // If no values are given calculate averages on the lattice
  {
    for(fld=0;fld<nflds;fld++)
    {
      fldsqrd[fld]=0.;
      fldvalue[fld]=0;
      LOOP
      {
        fldsqrd[fld]+=pw2(FIELD(fld));
        fldvalue[fld]+=FIELD(fld);
      }

      fldsqrd[fld] /= (float)gridsize;
      fldvalue[fld] /= (float)gridsize;
    }
  }
  else // If field values are given then use them instead
  {
    for(fld=0;fld<nflds;fld++)
    {
      fldsqrd[fld]=pw2(field_values[fld]);
      fldvalue[fld]=field_values[fld];
    }
  }
  
  mass_sq[0] = 3.*A3*fldsqrd[0] - 2.*A32*fldvalue[0];
  // Avoid tachyonic mass term
  if(mass_sq[0]<0.)
    mass_sq[0]=0.;
    
  // Put in scale factor correction. This calculation should be the same for all models.
  if(expansion>0) // If there's no expansion don't bother with this.
  {
    correction = pow(a,2.*rescale_s+2.);
    for(fld=0;fld<nflds;fld++)
      mass_sq[fld] *= correction;
  }
}

////////////////////////////////////////////////////////////////////////////////////
// Model-specific output functions - The function model_output is called externally/
////////////////////////////////////////////////////////////////////////////////////

inline void model_output(int flush,char *ext_)
{
  // Value by which to rescale the field to recover physical units
  rescaling=1./pow(a,rescale_r);
}

#undef PHI
#undef SOURCETERM
#undef A3
#undef A32

