/*
MODEL.H file for the SOLAR model.
Written by Gary Felder (gfelder@physics.stanford.edu)
Last Modified 4/19/01

This file contains the model specific functions and definitions for the model:
V = 0
where the sound speed is a function of position. Note that this requires modifications to the program files.

No rescaling is done. Distances are measured in Mm and time in units of 100 s. (This choice reflects the unperturbed sound speed, which is taken to be 10 km/s.)
*/

/*
General comments about the model.h file:
This file contains the following functions - all called externally
modelinfo(FILE *info_) outputs information about the model and model-specific parameters to a file.
modelinitialize() performs any model-specific initialization
potential_energy(int term, float *field_values) calculates the average potential energy density term by term. The variable num_potential_terms (just above this function) specifies how many separate potential terms are used in this model.
dvdf(int fld, int i, int j, int k) calculates the potential term in the equation of motion, dV/dfield, for the field fld at the lattice point (i,j,k)
effective_mass(float mass_sq[], float *field_values) calculates the square masses of the fields and puts them in the array mass_sq. The parameter beginning tells the function to use initial field values - if this parameter is zero then the field quantities will be calculated dynamically.
model_output(int flush, char *ext_) allows each model to include its own specialized output function(s). The parameter flush is set to 1 when infrequent calculations are being performed and 0 otherwise. The string ext_ gives the extension for output filenames.
*/

/* - This section should be copied into parameters.h when using this model
// ---Adjustable parameters for SOLAR model--- //
const float omega0 = 1.88; // Central frequency of the wave packet
const float tau = 5.; // Width of the wave packet in time
const float R = 10.; // Radius of the occlusion
const float A=.02; // Amplitude of the sound speed perturbation
const float xdist = 90.; // Distance from source to receiver
const float minusts = tau; // Time since the source perturbation ended
const int sreceiver=1; // Whether to output the field value at the receiver
*/
// Derived parameter(s)
const int ndist=(int)(xdist/dx); // Number of grid spacings between source and receiver

// Rescaling parameters.
// The program variables ("_pr") are defined as
//   f_pr = rescale_A a^rescale_r f (f=field value)
//   x_pr = rescale_B x (x=distance)
//   dt_pr = rescale_B a^rescale_s dt (t=time)
// The constants beta, cpl, and f0 are used to set the variable rescalings rescale_A, rescale_B, rescale_r, and rescale_s.
// These rescaling constants may be reset independently; their settings in terms of beta, cpl, and f0 are suggestions. See the documentation for more details.
// These rescalings are intrinsic to the model and probably shouldn't be changed for individual runs. Adjustable parameters are stored in "parameters.h"
const float beta=1.; // Exponent of the dominant term in the potential
const float cpl=1.; // Coefficient of the dominant term in the potential (up to numerical factors - see documentation)
const float f0=0.; // Initial value of phi in Planck units, typically the point at which phi'=0
// By default these are automatically set to A=1/f0, B=sqrt(cpl) f0^(-1+beta/2), R=6/(2+beta), S=3(2-beta)/(2+beta). They may be adjusted to different values, but the relationship S=2R-3 must be maintained for the program equations to remain correct.
const float rescale_A=1.;
const float rescale_B=1.;
const float rescale_r=0.;
// The value of S in terms of R SHOULD NOT be changed.
const float rescale_s=2.*rescale_r-3.;

// Other global variables
// The array model_vars is intended to hold any model-specific, non-constant global variables.
// These variables should be initialized in modelinitialize() below
// Even if you're not using any, num_model_vars should be at least 1 to keep some compilers happy.
const int num_model_vars=1;
// Model specific variables: None are defined for this model.
extern float model_vars[num_model_vars];

// Model specific details about the run to be output to an information file
inline void modelinfo(FILE *info_)
{
  // Name and description of model
  fprintf(info_,"Solar Model\n");
  fprintf(info_,"V = Acoustic Wave Scattering Around a Sound Speed Perturbation\n\n");

  // Model specific parameter values
  fprintf(info_,"omega0 = %f\n",omega0);
  fprintf(info_,"tau = %f\n",tau);
  fprintf(info_,"R = %f\n",R);
  fprintf(info_,"A = %f\n",A);
  fprintf(info_,"|xs - xr| = %f\n",xdist);
  fprintf(info_,"ts = %f\n",-minusts);
}

// The next three functions - distance, omega, and create_source - are specific to the initialization for this model
// Find the distance between two gridpoints in one dimension in units of grid spacing 
inline float distance(int a, int b)
{
  int x = abs(a-b);
  if(x < N-x)
    return( (float)x );
  else
    return( (float)(N-x) );
}
// Amplitude of field initialization
inline float omega(float x)
{
  return ( cos(omega0*x)/exp(pw2(x/tau)) );
}
// Initialize the source centered at (x,y,z)
// This function ADDS the source to whatever field values are already present
inline void create_source(int x, int y, int z)
{
  int i,j,k;
  float norm=1./4./pi; // Normalization of field values
  float dist;
  LOOP
  {
    if(NDIMS==1)
      dist = dx*distance(i,x);
    else if(NDIMS==2)
      dist = dx*sqrt(pw2(distance(i,x)) + pw2(distance(j,y)));
    else if(NDIMS==3)
      dist = dx*sqrt(pw2(distance(i,x)) + pw2(distance(j,y)) + pw2(distance(k,z)));
    // Kludge to avoid dividing by zero
    if(dist==0.)
      dist=dx;
    FIELD(0) += norm/dist*omega(minusts - dist);
  }
}
// Initialize the sound speed for a perturbation centered at (x,y,z)
// This function ADDS the sound speed perturbation to whatever values are already present
inline void create_soundspeed(int x, int y, int z)
{
  int i,j,k;
  float dist;
  LOOP
  {
    if(NDIMS==1)
      dist = dx*distance(i,x);
    else if(NDIMS==2)
      dist = dx*sqrt(pw2(distance(i,x)) + pw2(distance(j,y)));
    else if(NDIMS==3)
      dist = dx*sqrt(pw2(distance(i,x)) + pw2(distance(j,y)) + pw2(distance(k,z)));
    if(dist<=R)
      SOUNDSPEED += A;
  }
}

// Perform any model specific initialization
// This function is called twice, once before initializing the fields (which_call=1) and once after (which_call=2)
inline void modelinitialize(int which_call)
{
  DECLARE_INDICES

  if(which_call==1)
  {
    // Check parameters
    if(nflds!=1) // Check number of fields
    {
      printf("Number of fields for COLEMANWEINBERG model must be 1. Exiting.\n");
      exit(1);
    }
    if(N<=ndist) // Check size of grid
    {
      printf("Grid isn't large enough for distance from source to receiver. Exiting\n");
      exit(1);
    }
    if(minusts<tau) // Check time since source
      printf("Warning: The time since the source peak (-ts) is smaller than the source width (tau)\n");
    if(expansion!=0)
    {
      printf("Solar model is only designed to run without expansion. Exiting\n");
      exit(1);
    }

    // Initialize fields
    LOOP // Initialize field values to zero
      FIELD(0)=0.;
    if(NDIMS==1) // Center source at edge of lattice
      create_source(0,0,0);
    else if(NDIMS==2) // Center source on one edge
      create_source(ndist/2,0,0);
    else if(NDIMS==3) // For three dimensions center the source on the slice i=0 so it can be seen on a 2D slice
      create_source(0,ndist/2,0);
    // Prevent the usual initialization
    no_initialization=1;

    // Initialize sound speed array
    LOOP // Initialize sound speed to 1
      SOUNDSPEED=1.;
    if(NDIMS==1)
      create_soundspeed(ndist/2,0,0);
    else if(NDIMS==2)
      create_soundspeed(ndist/2,ndist/2,0);
    else if(NDIMS==3)
      create_soundspeed(0,ndist/2,ndist/2);
  }
}

// The constant num_potential_terms must be defined for use by outside functions
// Terms: None
const int num_potential_terms=0; // Number of terms that are calculated separately in the potential
// Potential energy terms
// See documentation for normalization of these terms.
// When setting initial conditions field values will be supplied in the array field_values. Otherwise this function will calculate them on the lattice.
inline float potential_energy(int term, float *field_values)
{
  return (0.);
}

// Potential terms in the equations of motion, dV/dfield, evaluated at point (i,j,k)
// See documentation for details on the normalization of these terms
inline float dvdf(int fld, INDEXLIST)
{
  return(0.);
}

// Calculate effective mass squared and put it into the array mass_sq[] (used for initial conditions and power spectra)
// See documentation for normalization of these terms
// When setting initial conditions field values will be supplied in the array field_values. Otherwise this function will calculate them on the lattice.
inline void effective_mass(float mass_sq[], float *field_values)
{
  int fld;
  for(fld=0;fld<nflds;fld++)
    mass_sq[fld]=0.;
}

////////////////////////////////////////////////////////////////////////////////////
// Model-specific output functions - The function model_output is called externally/
////////////////////////////////////////////////////////////////////////////////////

// Output the field value at the receiver
inline void receiver(int flush, char *ext_)
{
  static FILE *receiver_;
  char name_[500];

  static int first=1;
  if(first==1)
  {
    sprintf(name_,"receiver%s",ext_);
    receiver_=fopen(name_,"w");
    first=0;
  }

  // Output field value
  // This point will be at the center of the lattice for 1D or 2D and at the center of the i=0 slice for 3D
  fprintf(receiver_,"%f %f\n",t,FIELDPOINT(0,0,ndist/2,ndist));

  if(flush)
    fflush(receiver_);
}

inline void model_output(int flush,char *ext_)
{
  if(sreceiver==1)
    receiver(flush, ext_);
}
